//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of CPCtelera: An Amstrad CPC Game Engine
//  Copyright (C) 2015 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <cpctelera.h>
#include "player8bit.h"
#include "map1.h"
#include "map2.h"
#include "map3.h"
#include "map4.h"
#include "Tilemap.h"
#include "Logotipo.h"
#include "ataque.h"
#include "minimapa1.h"
#include "heart.h"

__at(0x200) const unsigned char music[127];



//#############
//# ESTADOS DEL JUEGO #
//#############
typedef struct{
  u8 estado;
} GEstados;

enum {
  P_premenu = 0,
  P_menu,
  P_prejuego,
  P_juego,
  P_numPantallas
} EPantallas;


GEstados game;

u8* mapa;

//PANTALLA
#define ORIGEN_MAPA_Y 20
#define  ORIGEN_MAPA cpctm_screenPtr(CPCT_VMEM_START,0,ORIGEN_MAPA_Y)
#define  GNRL_WIDTH 80
#define  GNRL_HEIGHT 200
#define  ON 1
#define OFF 0


typedef struct {
  u8 opcion;
  u8 audio;
  u8 pvez;
} Tmenu;
Tmenu Gmenu;
//PERSONAJE
#define  NO 0
#define  SI 1
#define  VELX 1
#define  VELY 2
#define  PLAYER_FRAMES  12
#define  ANIM_PAUSA 6
#define  ANIM_ANDAR 3
#define  ANIM_ATTACK 3
#define NUM_VIDAS 3
#define LIFE_FRAMES 2

//MAPAS
#define MAPAS 4

typedef struct {
  u8* mapa;
  u8 x,y;
} Tmapa;

//array de MAPAS
const Tmapa nivel1[MAPAS] = {
  {map1,40,100}, {map2,40,160},
  {map3,70,150}, {map4,5,130},
};
//u8* const arrayMapas[num] = {map1, map2, map3, map4};
u8 map = 0; //mapa actual
u8 pmap = 0;
//########################
//ESTRUCTURA DEL PERSONAJE
//########################

//TABLA TRANSPARENCIA
cpctm_createTransparentMaskTable (player_trans, 0x0100, M0, 0);

//FRAMES
typedef struct {
   u8* sprite;
} TFrame;

//ATAQUE
typedef struct{
  u8 x,y;
  u8 px,py;
  u8* frame;
  u8 mirar;
} TAttack;
const TAttack Ainicio ={
  0,0,0,0,
  sprite_ataque,0
};
TAttack ataque;

//TIPO PLAYER
typedef struct  {
  u8 x,y;
  u8 px,py;
  u8 mover;
  u8 estado;
  u8 nvidas;
  u8 mirar;
  u8 nframe;
  TFrame* frame;
} Player;
const Player Pinicio = {
  40, 100,
  40, 100,
  0, 0,
  NUM_VIDAS,
  0,0,
  &player_frames[0]
};
Player jugador;
enum{
  M_abajo = 0,
  M_izquierda,
  M_derecha,
  M_arriba,
  M_numVistas
}EMirar;
enum{
  E_parar = 0,
  E_mover,
  E_atacar,
  E_numEventos
}EEventos;
enum{
  ST_quieto = 0,
  ST_movimiento,
  ST_atacando,
  ST_numEstados
}EEstados;

//PLAYER FRAMES
const TFrame player_frames[PLAYER_FRAMES] = {
      {sprite_player8bit_00}   ,  {sprite_player8bit_01}
   ,  {sprite_player8bit_02}   ,  {sprite_player8bit_03}
   ,  {sprite_player8bit_04}   ,  {sprite_player8bit_05}
   ,  {sprite_player8bit_06}   ,  {sprite_player8bit_07}
   ,  {sprite_player8bit_08}   ,  {sprite_player8bit_09}
   ,  {sprite_player8bit_10}   ,  {sprite_player8bit_11}
};
const TFrame life_frames[LIFE_FRAMES] = {{sprite_heart_0},{sprite_heart_1}};


//#############
//# FUNCIONES #
//#############

void dibujarLogo(){
  cpct_drawSprite(logo_Logotipo_0,
                  cpctm_screenPtr(CPCT_VMEM_START,(GNRL_WIDTH/2)-LOGO_LOGOTIPO_0_W, 0),
                  LOGO_LOGOTIPO_0_W, LOGO_LOGOTIPO_0_H);
  cpct_drawSprite(logo_Logotipo_1,
                  cpctm_screenPtr(CPCT_VMEM_START, (GNRL_WIDTH/2), 0),
                  LOGO_LOGOTIPO_1_W, LOGO_LOGOTIPO_1_H);
}
void playmusic() {
   __asm
      exx
      .db #0x08
      push af
      push bc
      push de
      push hl
      call _cpct_akp_musicPlay
      pop hl
      pop de
      pop bc
      pop af
      .db #0x08
      exx
   __endasm;
}
void dibujarPlayer(){
  u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START,jugador.x,jugador.y);
  cpct_drawSpriteMaskedAlignedTable (jugador.frame->sprite, pvmem,SPRITE_PLAYER8BIT_00_W,SPRITE_PLAYER8BIT_00_H, player_trans);
}
void borrarPlayer(){
  u8 w = 2 + (jugador.px & 1);
  u8 h = 4 + (jugador.py & 3 ? 1 : 0);
  cpct_etm_drawTileBox2x4(jugador.px/2,(jugador.py- ORIGEN_MAPA_Y)/4,w,h,map1_W,ORIGEN_MAPA,mapa);
}
void SpritePlayer(){
    borrarPlayer();
    jugador.px = jugador.x;
    jugador.py = jugador.y;
    dibujarPlayer();
}

void dibujarMiniMapa(){
    u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START,GNRL_WIDTH-MAP_MINIMAPA1_W,0);
    cpct_drawSprite (map_minimapa1,pvmem,MAP_MINIMAPA1_W,MAP_MINIMAPA1_H);
}
void dibujarVida(){
    u8 i;
    for (i = 0; i < NUM_VIDAS; i++) {
        u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START,i*SPRITE_HEART_0_W,0);
        if(jugador.nvidas>i){
          cpct_drawSprite (life_frames[0].sprite,pvmem,SPRITE_HEART_0_W,SPRITE_HEART_0_H);
        }else{
          cpct_drawSprite (life_frames[1].sprite,pvmem,SPRITE_HEART_0_W,SPRITE_HEART_0_H);
        }
    }
}
void dibujarHUD(){
  dibujarVida();
  dibujarMiniMapa();
}
void dibujarAtaque(){
  u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START,ataque.x,ataque.y);
  cpct_drawSpriteMaskedAlignedTable (ataque.frame, pvmem,SPRITE_ATAQUE_W,SPRITE_ATAQUE_H, player_trans);
}
void borrarAtaque(){
  u8 w = 2 + (ataque.px & 0);
  u8 h = 2 + (ataque.py & 2 ? 1 : 0);
  cpct_etm_drawTileBox2x4(ataque.px/2,(ataque.py-20)/4,w,h,map1_W,ORIGEN_MAPA,mapa);
}
void SpriteAtaque(){
  borrarAtaque();
  ataque.px = ataque.x;
  ataque.py = ataque.y;
  dibujarAtaque();
}
void changeState(){
  borrarAtaque();
  if(jugador.mover==SI){
    jugador.estado = ST_movimiento;
  }else{
    jugador.estado = ST_quieto;
  }
}
void AtaqueAbajo(){
  if(ataque.y+SPRITE_ATAQUE_H+1<GNRL_HEIGHT){
    ataque.y+=2;
  }else{
    changeState();
  }
}
void AtaqueArriba(){
  if(ataque.y>ORIGEN_MAPA_Y){
    ataque.y-=2;
  }else{
    changeState();
  }
}
void AtaqueDerecha(){
  if(ataque.x+SPRITE_ATAQUE_W<GNRL_WIDTH){
    ataque.x++;
  }else{
    changeState();
  }
}
void AtaqueIzquierda(){
  if(ataque.x>2){
    ataque.x--;
  }else{
    changeState();
  }
}

void UpdateAtaque(){
  switch (ataque.mirar) {
    case M_abajo:AtaqueAbajo();break;
    case M_izquierda:AtaqueIzquierda();break;
    case M_derecha:AtaqueDerecha();break;
    case M_arriba:AtaqueArriba();break;
  }
}

void dibujarMapa(){
   cpct_etm_drawTilemap2x4 (map1_W, map1_H, ORIGEN_MAPA, mapa);
}

//para cambiar de mapa
void avanzarMapa(u8 num) __z88dk_fastcall{
  mapa = nivel1[num].mapa; //avanza 1
  jugador.x = jugador.px = nivel1[num].x;
  jugador.y = jugador.py = nivel1[num].y;
  game.estado = P_prejuego;
}

//cojo la posicion del pixel de player y me dice en que posicion del array del mapa esta
u8* obtenerTilePtr(u8 x, u8 y) {
  return mapa + (y-ORIGEN_MAPA_Y)/4 * map1_W + x/2;
}

void moverDerecha(){
  u8* tile = obtenerTilePtr(jugador.x+4, jugador.y);
  u8* tile2 = obtenerTilePtr(jugador.x+4, jugador.y+15);
  u8* tile3 = obtenerTilePtr(jugador.x+4, jugador.y+5);

  if(*tile != 169 && *tile2 != 169 && *tile3 != 169 && *tile != 240 && *tile2 != 240 && *tile3 != 240 && *tile != 241 && *tile2 != 241 && *tile3 != 241 && *tile != 232 && *tile2 != 232 && *tile2 != 232 && *tile != 228 && *tile2 != 228 && *tile3 != 228 && *tile != 229 && *tile2 != 229 && *tile3 != 229 && *tile != 128 && *tile2 != 128 && *tile3 != 128 && *tile != 144 && *tile2 != 144 && *tile3 != 144 && *tile != 249 && *tile2 != 249 && *tile3 != 249 && *tile != 79 && *tile2 != 79 && *tile3 != 79 && *tile != 166 && *tile2 != 166 && *tile3 != 166 && *tile != 20 && *tile2 != 20 && *tile3 != 20 && *tile != 46 && *tile2 != 46 && *tile3 != 46){
    jugador.mirar = M_derecha;
    if(jugador.x+SPRITE_PLAYER8BIT_00_W<GNRL_WIDTH)
      jugador.x = jugador.x + VELX;
  }else if(*tile == 165 || *tile2 == 165 || *tile3 == 165){
    map = 3;
  }else if(map == 2 && (*tile == 246 || *tile2 == 246 || *tile3 == 246)){
    map = 0;
  }
}
void moverIzquierda(){
  u8* tile = obtenerTilePtr(jugador.x-1, jugador.y);
  u8* tile2 = obtenerTilePtr(jugador.x-1, jugador.y+15);
  u8* tile3 = obtenerTilePtr(jugador.x-1, jugador.y+5);

  if(*tile != 169 && *tile2 != 169 && *tile3 != 169 && *tile != 240 && *tile2 != 240 && *tile3 != 240 && *tile != 241 && *tile2 != 241 && *tile3 != 241 && *tile != 232 && *tile2 != 232 && *tile2 != 232 && *tile != 228 && *tile2 != 228 && *tile3 != 228 && *tile != 229 && *tile2 != 229 && *tile3 != 229 && *tile != 128 && *tile2 != 128 && *tile3 != 128 && *tile != 144 && *tile2 != 144 && *tile3 != 144 && *tile != 249 && *tile2 != 249 && *tile3 != 249 && *tile != 79 && *tile2 != 79 && *tile3 != 79 && *tile != 166 && *tile2 != 166 && *tile3 != 166 && *tile != 20 && *tile2 != 20 && *tile3 != 20 && *tile != 46 && *tile2 != 46 && *tile3 != 46){
    jugador.mirar = M_izquierda;
    if (jugador.x>0)
      jugador.x = jugador.x - VELX;
  }else if(*tile == 165 || *tile2 == 165 || *tile3 == 165){
    map = 2;
  }else if(map == 3 && (*tile == 194 || *tile2 == 194 || *tile3 == 194)){
    map = 0;
  }
}
void moverArriba(){
  u8* tile = obtenerTilePtr(jugador.x, jugador.y-1);
  u8* tile2 = obtenerTilePtr(jugador.x+3, jugador.y-1);
  u8* tile3 = obtenerTilePtr(jugador.x+1, jugador.y-1);

  if(*tile != 169 && *tile2 != 169 && *tile3 != 169 && *tile != 240 && *tile2 != 240 && *tile3 != 240 && *tile != 241 && *tile2 != 241 && *tile3 != 241 && *tile != 232 && *tile2 != 232 && *tile2 != 232 && *tile != 228 && *tile2 != 228 && *tile3 != 228 && *tile != 229 && *tile2 != 229 && *tile3 != 229 && *tile != 128 && *tile2 != 128 && *tile3 != 128 && *tile != 144 && *tile2 != 144 && *tile3 != 144 && *tile != 249 && *tile2 != 249 && *tile3 != 249 && *tile != 79 && *tile2 != 79 && *tile3 != 79 && *tile != 166 && *tile2 != 166 && *tile3 != 166 && *tile != 20 && *tile2 != 20 && *tile3 != 20 && *tile != 46 && *tile2 != 46 && *tile3 != 46){
    jugador.mirar = M_arriba;
    if (jugador.y>ORIGEN_MAPA_Y)
      jugador.y = jugador.y - VELY;
  }else if(*tile == 165 || *tile2 == 165 || *tile3 == 165){
    map = 1;
  }
}
void moverAbajo(){
  u8* tile = obtenerTilePtr(jugador.x, jugador.y+16);
  u8* tile2 = obtenerTilePtr(jugador.x+3, jugador.y+16);
  u8* tile3 = obtenerTilePtr(jugador.x+1, jugador.y+16);

  if(*tile != 169 && *tile2 != 169 && *tile3 != 169 && *tile != 240 && *tile2 != 240 && *tile3 != 240 && *tile != 241 && *tile2 != 241 && *tile3 != 241 && *tile != 232 && *tile2 != 232 && *tile2 != 232 && *tile != 228 && *tile2 != 228 && *tile3 != 228 && *tile != 229 && *tile2 != 229 && *tile3 != 229 && *tile != 128 && *tile2 != 128 && *tile3 != 128 && *tile != 144 && *tile2 != 144 && *tile3 != 144 && *tile != 249 && *tile2 != 249 && *tile3 != 249 && *tile != 79 && *tile2 != 79 && *tile3 != 79 && *tile != 166 && *tile2 != 166 && *tile3 != 166 && *tile != 20 && *tile2 != 20 && *tile3 != 20 && *tile != 46 && *tile2 != 46 && *tile3 != 46){
    jugador.mirar = M_abajo;
    if(jugador.y+SPRITE_PLAYER8BIT_00_H+VELY<GNRL_HEIGHT)
      jugador.y = jugador.y + VELY;
  }else if(map == 1 && (*tile == 68 || *tile2 == 68 || *tile3 == 68)){
    map = 0;
  }
}
u8 compruebaTeclas(const cpct_keyID* k, u8 numk) {
   u8 i;
   if (cpct_isAnyKeyPressed()) {
      for(i=1; i <= numk; i++, k++) {
         if (cpct_isKeyPressed(*k))
            return i;
      }
   }
   return 0;
}
void mirarA(u8 donde) __z88dk_fastcall{
  switch (donde) {
    case M_abajo:jugador.frame=&player_frames[0];break;
    case M_derecha:jugador.frame=&player_frames[3];break;
    case M_izquierda:jugador.frame=&player_frames[6];break;
    case M_arriba:jugador.frame=&player_frames[9];break;
  }
}
void animAndar(){
  jugador.nframe++;
  if(jugador.nframe==ANIM_PAUSA*ANIM_ANDAR){
    jugador.nframe=0;
  }else if(jugador.nframe % ANIM_PAUSA == 0){
    jugador.frame=&player_frames[ANIM_ANDAR*jugador.mirar+jugador.nframe/ANIM_PAUSA];
  }
}

void atacar(){
  ataque.mirar = jugador.mirar;
  switch (ataque.mirar) {
    case M_abajo:
    ataque.x = ataque.px = jugador.x-1+SPRITE_PLAYER8BIT_00_W/2;
    ataque.y = ataque.py = jugador.y+SPRITE_PLAYER8BIT_00_H+1;
          break;
    case M_izquierda:
    ataque.x = ataque.px = jugador.x-SPRITE_ATAQUE_W-4;
    ataque.y = ataque.py = jugador.y+SPRITE_PLAYER8BIT_00_H/2;
          break;
    case M_derecha:
    ataque.x = ataque.px = jugador.x+SPRITE_PLAYER8BIT_00_W+1;
    ataque.y = ataque.py = jugador.y+SPRITE_PLAYER8BIT_00_H/2;
          break;
    case M_arriba:
    ataque.x = ataque.px = jugador.x-1+SPRITE_PLAYER8BIT_00_W/2;
    ataque.y = ataque.py = jugador.y-SPRITE_ATAQUE_H;
          break;
  }
  jugador.estado = ST_atacando;
}

u8 teclas(){
  static const cpct_keyID keys[7] = {Key_CursorDown,Key_CursorRight,Key_CursorLeft,Key_CursorUp,Key_A,Key_Esc,Key_Return};
  u8 k = compruebaTeclas(keys, 7);
  return k;
}
void andando(){
  u8 i = teclas();
  if(i>0 && i<5){jugador.mover = SI;}
  switch (i) {
    case 0:jugador.estado=ST_quieto;break;
    case 1:moverAbajo();animAndar();break;
    case 2:moverDerecha();animAndar();break;
    case 3:moverIzquierda();animAndar();break;
    case 4:moverArriba();animAndar();break;
    //case 6:game.estado = P_premenu;break;
  }
}

void atacando(){
  u8 i = teclas();
  if(i>0 && i<5){jugador.mover = SI;}
  UpdateAtaque();
  switch (i) {
    case 0:break;
    case 1:moverAbajo();animAndar();break;
    case 2:moverDerecha();animAndar();break;
    case 3:moverIzquierda();animAndar();break;
    case 4:moverArriba();animAndar();break;
    case 5:break;//NO PUEDE VOLVER ATACAR
  }

}

void quieto(){
  u8 i = teclas();
  if(i>0 && i<5){jugador.mover = SI;}
  switch (i) {
    case 0:break;//NO OCURRE NADA
    case 1:mirarA(M_abajo);jugador.estado=ST_movimiento;break;
    case 2:mirarA(M_izquierda);jugador.estado=ST_movimiento;break;
    case 3:mirarA(M_derecha);jugador.estado=ST_movimiento;break;
    case 4:mirarA(M_arriba);jugador.estado=ST_movimiento;break;
    case 5:atacar();break;
    case 6:game.estado = P_premenu;break;
  }
}
void PlayerUpdate(){
  switch (jugador.estado) {
    case ST_quieto:quieto();break;
    case ST_movimiento:andando();break;
    case ST_atacando:atacando();break;
  }
}
void interrupcion() {
   static u8 kk;
   kk++;
   if (kk%6==0) {
      if(Gmenu.audio==ON)
        playmusic();
      cpct_scanKeyboard_if();
      kk = 0;
   }
}
void inicializar(){
  cpct_disableFirmware();
  cpct_setVideoMode(0);
  cpct_setBorder(HW_BLACK);
  cpct_setPalette(sprite_palette,14);
  Gmenu.opcion = 0;
  Gmenu.audio = ON;
  Gmenu.pvez = 0;
  mapa = map1;
  cpct_etm_setTileset2x4 (tile_tileset);
  //######################
  //# FUNCIONES DE JUEGO #
  //######################
  game.estado = P_premenu;
  cpct_setInterruptHandler(interrupcion);
  cpct_akp_musicInit(music);

  //###################
  //INICIALIZAR JUGADOR
  //###################
  cpct_memcpy((void*)&jugador, &Pinicio, sizeof(Player));
  cpct_memcpy((void*)&ataque, &Ainicio, sizeof(TAttack));
}

void juego(){
  PlayerUpdate();
  cpct_waitVSYNC();
  if(map!=pmap){
    avanzarMapa(map);
    pmap=map;
  }else{
    if(jugador.mover){
      SpritePlayer();
      jugador.mover = NO;
    }
    if(jugador.estado==ST_atacando){
      SpriteAtaque();
    }
  }
}
void prejuego(){
  game.estado = P_juego;
  cpct_waitVSYNC();
  cpct_clearScreen(0);
  dibujarHUD();
  dibujarMapa();
  dibujarPlayer();
}

void premenu(){
  game.estado = P_menu;
  cpct_waitVSYNC();
  cpct_clearScreen(0); //para brrar la primera pantalla q sale
}
void StartGame(){
  game.estado = P_prejuego;
  Gmenu.pvez = 1;
}
void Music(){
  Gmenu.audio = !Gmenu.audio;
  switch (Gmenu.audio) {
    case ON:cpct_akp_musicInit(music);break;
    case OFF:cpct_akp_stop();break;
  }
    cpct_akp_stop();
}
void Opciones(){
  switch (Gmenu.opcion) {
    case 0:StartGame();break;
    case 1:Music();break;
  }
}
void select(){
  u8 i = teclas();
  switch (i) {
    case 1:if(Gmenu.opcion==0){Gmenu.opcion++;}break;//ABAJO
    case 4:if(Gmenu.opcion==1){Gmenu.opcion--;}break;//ARRIBA
    case 7:Opciones();break;//EJECUTAR
  }
}
void DrawSelection(){
  u8* memptr = cpct_getScreenPtr(CPCT_VMEM_START, 10, 100);
  u8* memptr2 = cpct_getScreenPtr(CPCT_VMEM_START, 10, 120);
  switch (Gmenu.opcion) {
    case 0:
    if (Gmenu.pvez==0) {
      cpct_drawStringM0("NEW GAME", memptr, 8, 14);
    }else
      cpct_drawStringM0("CONTINUE GAME", memptr, 8, 14);
    if(!Gmenu.audio)
      cpct_drawStringM0("AUDIO : OFF", memptr2, 8, 0);
    else
      cpct_drawStringM0("AUDIO : ON", memptr2, 8, 0);
    break;
    case 1:
    if (Gmenu.pvez==0) {
      cpct_drawStringM0("NEW GAME", memptr, 8, 0);
    }else
      cpct_drawStringM0("CONTINUE GAME", memptr, 8, 0);
    if(!Gmenu.audio)
      cpct_drawStringM0("AUDIO : OFF", memptr2, 8, 14);
    else
      cpct_drawStringM0("AUDIO : ON ", memptr2, 8, 14);
    break;
  }

}
void menu(){
  select();
  cpct_waitVSYNC();
  dibujarLogo();
  DrawSelection();
}
void main() {
  inicializar();
   // Loop forever
   while (1){
     switch (game.estado) {
       case P_premenu:premenu();break;
       case P_menu:menu();break;
       case P_prejuego:prejuego();break;
       case P_juego:juego();break;
     }
   }
}
